@@ -6,24 +6,25 @@ use tracing::{debug, warn};
 
 use uv_cache::{Cache, CacheBucket, WheelCache};
 use uv_cache_info::Timestamp;
-use uv_configuration::{BuildOptions, ConfigSettings, PackageConfigSettings, Reinstall};
+use uv_configuration::{BuildOptions, Reinstall};
 use uv_distribution::{
     BuiltWheelIndex, HttpArchivePointer, LocalArchivePointer, RegistryWheelIndex,
 };
 use uv_distribution_filename::WheelFilename;
 use uv_distribution_types::{
-    BuiltDist, CachedDirectUrlDist, CachedDist, Dist, Error, ExtraBuildRequires,
-    ExtraBuildVariables, Hashed, IndexLocations, InstalledDist, Name, RequirementSource,
-    Resolution, ResolvedDist, SourceDist,
+    BuiltDist, CachedDirectUrlDist, CachedDist, ConfigSettings, Dist, Error, ExtraBuildRequires,
+    ExtraBuildVariables, Hashed, IndexLocations, InstalledDist, Name, PackageConfigSettings,
+    RequirementSource, Resolution, ResolvedDist, SourceDist,
 };
 use uv_fs::Simplified;
+use uv_normalize::PackageName;
 use uv_platform_tags::{IncompatibleTag, TagCompatibility, Tags};
 use uv_pypi_types::VerbatimParsedUrl;
 use uv_python::PythonEnvironment;
 use uv_types::HashStrategy;
 
-use crate::SitePackages;
 use crate::satisfies::RequirementSatisfaction;
+use crate::{InstallationStrategy, SitePackages};
 
 /// A planner to generate an [`Plan`] based on a set of requirements.
 #[derive(Debug)]
@@ -51,6 +52,7 @@ impl<'a> Planner<'a> {
     pub fn build(
         self,
         mut site_packages: SitePackages,
+        installation: InstallationStrategy,
         reinstall: &Reinstall,
         build_options: &BuildOptions,
         hasher: &HashStrategy,
@@ -64,8 +66,16 @@ impl<'a> Planner<'a> {
         tags: &Tags,
     ) -> Result<Plan> {
         // Index all the already-downloaded wheels in the cache.
-        let mut registry_index =
-            RegistryWheelIndex::new(cache, tags, index_locations, hasher, config_settings);
+        let mut registry_index = RegistryWheelIndex::new(
+            cache,
+            tags,
+            index_locations,
+            hasher,
+            config_settings,
+            config_settings_package,
+            extra_build_requires,
+            extra_build_variables,
+        );
         let built_index = BuiltWheelIndex::new(
             cache,
             tags,
@@ -112,7 +122,17 @@ impl<'a> Planner<'a> {
                     [] => {}
                     [installed] => {
                         let source = RequirementSource::from(dist);
-                        match RequirementSatisfaction::check(installed, &source) {
+                        match RequirementSatisfaction::check(
+                            dist.name(),
+                            installed,
+                            &source,
+                            installation,
+                            tags,
+                            config_settings,
+                            config_settings_package,
+                            extra_build_requires,
+                            extra_build_variables,
+                        ) {
                             RequirementSatisfaction::Mismatch => {
                                 debug!(
                                     "Requirement installed, but mismatched:\n  Installed: {installed:?}\n  Requested: {source:?}"
@@ -124,6 +144,24 @@ impl<'a> Planner<'a> {
                             }
                             RequirementSatisfaction::OutOfDate => {
                                 debug!("Requirement installed, but not fresh: {installed}");
+
+                                // If we made it here, something went wrong in the resolver, because it returned an
+                                // already-installed distribution that we "shouldn't" use. Typically, this means the
+                                // distribution was considered out-of-date, but in a way that the resolver didn't
+                                // detect, and is indicative of drift between the resolver's candidate selector and
+                                // the install plan. For example, at present, the resolver doesn't check that an
+                                // installed distribution was built with the expected build settings. Treat it as
+                                // up-to-date for now; it's just means we may not rebuild a package when we otherwise
+                                // should. This is a known issue, but should only affect the `uv pip` CLI, as the
+                                // project APIs never return installed distributions during resolution (i.e., the
+                                // resolver is stateless).
+                                // TODO(charlie): Incorporate these checks into the resolver.
+                                if matches!(dist, ResolvedDist::Installed { .. }) {
+                                    warn!(
+                                        "Installed distribution was considered out-of-date, but returned by the resolver: {dist}"
+                                    );
+                                    continue;
+                                }
                             }
                             RequirementSatisfaction::CacheInvalid => {
                                 // Already logged
@@ -213,6 +251,7 @@ impl<'a> Planner<'a> {
                     match HttpArchivePointer::read_from(&cache_entry) {
                         Ok(Some(pointer)) => {
                             let cache_info = pointer.to_cache_info();
+                            let build_info = pointer.to_build_info();
                             let archive = pointer.into_archive();
                             if archive.satisfies(hasher.get(dist.as_ref())) {
                                 let cached_dist = CachedDirectUrlDist {
@@ -223,6 +262,7 @@ impl<'a> Planner<'a> {
                                     },
                                     hashes: archive.hashes,
                                     cache_info,
+                                    build_info,
                                     path: cache.archive(&archive.id).into_boxed_path(),
                                 };
 
@@ -286,6 +326,7 @@ impl<'a> Planner<'a> {
                             Ok(timestamp) => {
                                 if pointer.is_up_to_date(timestamp) {
                                     let cache_info = pointer.to_cache_info();
+                                    let build_info = pointer.to_build_info();
                                     let archive = pointer.into_archive();
                                     if archive.satisfies(hasher.get(dist.as_ref())) {
                                         let cached_dist = CachedDirectUrlDist {
@@ -296,6 +337,7 @@ impl<'a> Planner<'a> {
                                             },
                                             hashes: archive.hashes,
                                             cache_info,
+                                            build_info,
                                             path: cache.archive(&archive.id).into_boxed_path(),
                                         };
 
@@ -638,3 +680,76 @@ pub struct Plan {
     /// _not_ necessary to satisfy the requirements.
     pub extraneous: Vec<InstalledDist>,
 }
+
+impl Plan {
+    /// Returns `true` if the plan is empty.
+    pub fn is_empty(&self) -> bool {
+        self.cached.is_empty()
+            && self.remote.is_empty()
+            && self.reinstalls.is_empty()
+            && self.extraneous.is_empty()
+    }
+
+    /// Partition the remote distributions based on a predicate function.
+    ///
+    /// Returns a tuple of plans, where the first plan contains the remote distributions that match
+    /// the predicate, and the second plan contains those that do not.
+    ///
+    /// Any extraneous and cached distributions will be returned in the first plan, while the second
+    /// plan will contain any `false` matches from the remote distributions, along with any
+    /// reinstalls for those distributions.
+    pub fn partition<F>(self, mut f: F) -> (Self, Self)
+    where
+        F: FnMut(&PackageName) -> bool,
+    {
+        let Self {
+            cached,
+            remote,
+            reinstalls,
+            extraneous,
+        } = self;
+
+        // Partition the remote distributions based on the predicate function.
+        let (left_remote, right_remote) = remote
+            .into_iter()
+            .partition::<Vec<_>, _>(|dist| f(dist.name()));
+
+        // If any remote distributions are not matched, but are already installed, ensure that
+        // they're uninstalled as part of the right plan. (Uninstalling them as part of the left
+        // plan risks uninstalling them from the environment _prior_ to the replacement being built.)
+        let (left_reinstalls, right_reinstalls) = reinstalls
+            .into_iter()
+            .partition::<Vec<_>, _>(|dist| !right_remote.iter().any(|d| d.name() == dist.name()));
+
+        // If the right plan is non-empty, then remove extraneous distributions as part of the
+        // right plan, so they're present until the very end. Otherwise, we risk removing extraneous
+        // packages that are actually build dependencies.
+        let (left_extraneous, right_extraneous) = if right_remote.is_empty() {
+            (extraneous, vec![])
+        } else {
+            (vec![], extraneous)
+        };
+
+        // Always include the cached distributions in the left plan.
+        let (left_cached, right_cached) = (cached, vec![]);
+
+        // Include all cached and extraneous distributions in the left plan.
+        let left_plan = Self {
+            cached: left_cached,
+            remote: left_remote,
+            reinstalls: left_reinstalls,
+            extraneous: left_extraneous,
+        };
+
+        // The right plan will only contain the remote distributions that did not match the predicate,
+        // along with any reinstalls for those distributions.
+        let right_plan = Self {
+            cached: right_cached,
+            remote: right_remote,
+            reinstalls: right_reinstalls,
+            extraneous: right_extraneous,
+        };
+
+        (left_plan, right_plan)
+    }
+}
