@@ -9,12 +9,12 @@ use std::sync::LazyLock;
 use arcstr::ArcStr;
 use regex::Regex;
 use thiserror::Error;
-use url::{ParseError, Url};
+use url::Url;
 use uv_cache_key::{CacheKey, CacheKeyHasher};
 
 #[cfg_attr(not(feature = "non-pep508-extensions"), allow(unused_imports))]
 use uv_fs::{normalize_absolute_path, normalize_url_path};
-use uv_redacted::DisplaySafeUrl;
+use uv_redacted::{DisplaySafeUrl, DisplaySafeUrlError};
 
 use crate::Pep508Url;
 
@@ -57,12 +57,11 @@ impl VerbatimUrl {
     }
 
     /// Parse a URL from a string.
-    pub fn parse_url(given: impl AsRef<str>) -> Result<Self, ParseError> {
-        let url = Url::parse(given.as_ref())?;
-        Ok(Self {
-            url: DisplaySafeUrl::from(url),
-            given: None,
-        })
+    pub fn parse_url(given: impl AsRef<str>) -> Result<Self, VerbatimUrlError> {
+        let given = given.as_ref();
+        let url = DisplaySafeUrl::parse(given)?;
+
+        Ok(Self { url, given: None })
     }
 
     /// Convert a [`VerbatimUrl`] from a path or a URL.
@@ -189,10 +188,8 @@ impl VerbatimUrl {
         let (path, fragment) = split_fragment(path);
 
         // Convert to a URL.
-        let mut url = DisplaySafeUrl::from(
-            Url::from_file_path(path.clone())
-                .unwrap_or_else(|()| panic!("path is absolute: {}", path.display())),
-        );
+        let mut url = DisplaySafeUrl::from_file_path(path.clone())
+            .unwrap_or_else(|()| panic!("path is absolute: {}", path.display()));
 
         // Set the fragment, if it exists.
         if let Some(fragment) = fragment {
@@ -256,7 +253,7 @@ impl std::str::FromStr for VerbatimUrl {
     type Err = VerbatimUrlError;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        Ok(Self::parse_url(s).map(|url| url.with_given(s))?)
+        Self::parse_url(s).map(|url| url.with_given(s))
     }
 }
 
@@ -276,7 +273,7 @@ impl Deref for VerbatimUrl {
 
 impl From<Url> for VerbatimUrl {
     fn from(url: Url) -> Self {
-        Self::from_url(DisplaySafeUrl::from(url))
+        Self::from_url(DisplaySafeUrl::from_url(url))
     }
 }
 
@@ -395,7 +392,7 @@ impl Pep508Url for VerbatimUrl {
 pub enum VerbatimUrlError {
     /// Failed to parse a URL.
     #[error(transparent)]
-    Url(#[from] ParseError),
+    Url(#[from] DisplaySafeUrlError),
 
     /// Received a relative path, but no working directory was provided.
     #[error("relative path without a working directory: {0}")]
@@ -650,6 +647,8 @@ impl std::fmt::Display for Scheme {
 
 #[cfg(test)]
 mod tests {
+    use insta::assert_snapshot;
+
     use super::*;
 
     #[test]
@@ -754,4 +753,27 @@ mod tests {
         let url = Url::parse("https://github.com/pypa/pip/archive/1.3.1.zip#sha1=da9234ee9982d4bbb3c72346a6de940a148ea686").unwrap();
         assert!(!looks_like_git_repository(&url));
     }
+
+    #[test]
+    fn parse_url_ambiguous() {
+        assert_snapshot!(
+            VerbatimUrl::parse_url("https://user/name:password@domain/a/b/c").unwrap_err().to_string(),
+            @"ambiguous user/pass authority in URL (not percent-encoded?): https:***@domain/a/b/c"
+        );
+
+        assert_snapshot!(
+            VerbatimUrl::parse_url("https://user\\name:password@domain/a/b/c").unwrap_err().to_string(),
+            @"ambiguous user/pass authority in URL (not percent-encoded?): https:***@domain/a/b/c"
+        );
+
+        assert_snapshot!(
+            VerbatimUrl::parse_url("https://user#name:password@domain/a/b/c").unwrap_err().to_string(),
+            @"ambiguous user/pass authority in URL (not percent-encoded?): https:***@domain/a/b/c"
+        );
+
+        assert_snapshot!(
+            VerbatimUrl::parse_url("https://user.com/name:password@domain/a/b/c").unwrap_err().to_string(),
+            @"ambiguous user/pass authority in URL (not percent-encoded?): https:***@domain/a/b/c"
+        );
+    }
 }
