@@ -1122,6 +1122,8 @@ pub enum Source {
         rev: Option<String>,
         tag: Option<String>,
         branch: Option<String>,
+        /// Whether to use Git LFS when cloning the repository.
+        lfs: Option<bool>,
         #[serde(
             skip_serializing_if = "uv_pep508::marker::ser::is_empty",
             serialize_with = "uv_pep508::marker::ser::serialize",
@@ -1220,6 +1222,7 @@ impl<'de> Deserialize<'de> for Source {
             rev: Option<String>,
             tag: Option<String>,
             branch: Option<String>,
+            lfs: Option<bool>,
             url: Option<DisplaySafeUrl>,
             path: Option<PortablePathBuf>,
             editable: Option<bool>,
@@ -1243,6 +1246,7 @@ impl<'de> Deserialize<'de> for Source {
             rev,
             tag,
             branch,
+            lfs,
             url,
             path,
             editable,
@@ -1320,6 +1324,7 @@ impl<'de> Deserialize<'de> for Source {
                 rev,
                 tag,
                 branch,
+                lfs,
                 marker,
                 extra,
                 group,
@@ -1575,6 +1580,10 @@ pub enum SourceError {
         "`{0}` did not resolve to a Git repository, but a Git reference (`--branch {1}`) was provided."
     )]
     UnusedBranch(String, String),
+    #[error(
+        "`{0}` did not resolve to a Git repository, but a Git extension (`--lfs`) was provided."
+    )]
+    UnusedLfs(String),
     #[error(
         "`{0}` did not resolve to a local directory, but the `--editable` flag was provided. Editable installs are only supported for local directories."
     )]
@@ -1604,12 +1613,13 @@ impl Source {
         rev: Option<String>,
         tag: Option<String>,
         branch: Option<String>,
+        lfs: Option<bool>,
         root: &Path,
         existing_sources: Option<&BTreeMap<PackageName, Sources>>,
     ) -> Result<Option<Self>, SourceError> {
         // If the user specified a Git reference for a non-Git source, try existing Git sources before erroring.
         if !matches!(source, RequirementSource::Git { .. })
-            && (branch.is_some() || tag.is_some() || rev.is_some())
+            && (branch.is_some() || tag.is_some() || rev.is_some() || lfs.is_some())
         {
             if let Some(sources) = existing_sources {
                 if let Some(package_sources) = sources.get(name) {
@@ -1629,6 +1639,7 @@ impl Source {
                                 rev,
                                 tag,
                                 branch,
+                                lfs,
                                 marker: *marker,
                                 extra: extra.clone(),
                                 group: group.clone(),
@@ -1646,6 +1657,9 @@ impl Source {
             if let Some(branch) = branch {
                 return Err(SourceError::UnusedBranch(name.to_string(), branch));
             }
+            if let Some(true) = lfs {
+                return Err(SourceError::UnusedLfs(name.to_string()));
+            }
         }
 
         // If we resolved a non-path source, and user specified an `--editable` flag, error.
@@ -1754,6 +1768,7 @@ impl Source {
                         rev: rev.cloned(),
                         tag,
                         branch,
+                        lfs,
                         git: git.repository().clone(),
                         subdirectory: subdirectory.map(PortablePathBuf::from),
                         marker: MarkerTree::TRUE,
@@ -1765,6 +1780,7 @@ impl Source {
                         rev,
                         tag,
                         branch,
+                        lfs,
                         git: git.repository().clone(),
                         subdirectory: subdirectory.map(PortablePathBuf::from),
                         marker: MarkerTree::TRUE,
