@@ -23,18 +23,19 @@ use uv_distribution_types::{
 use uv_fs::Simplified;
 use uv_install_wheel::LinkMode;
 use uv_installer::{InstallationStrategy, SatisfiesResult, SitePackages};
-use uv_normalize::{DefaultExtras, DefaultGroups};
+use uv_normalize::{DefaultExtras, DefaultGroups, PackageName};
 use uv_preview::{Preview, PreviewFeatures};
 use uv_pypi_types::Conflicts;
 use uv_python::{
-    EnvironmentPreference, Prefix, PythonEnvironment, PythonInstallation, PythonPreference,
-    PythonRequest, PythonVersion, Target,
+    EnvironmentPreference, Prefix, PythonDownloads, PythonEnvironment, PythonInstallation,
+    PythonPreference, PythonRequest, PythonVersion, Target,
 };
 use uv_requirements::{GroupsSpecification, RequirementsSource, RequirementsSpecification};
 use uv_resolver::{
     DependencyMode, ExcludeNewer, FlatIndex, OptionsBuilder, PrereleaseMode, PylockToml,
     PythonRequirement, ResolutionMode, ResolverEnvironment,
 };
+use uv_settings::PythonInstallMirrors;
 use uv_torch::{TorchMode, TorchSource, TorchStrategy};
 use uv_types::HashStrategy;
 use uv_warnings::{warn_user, warn_user_once};
@@ -45,6 +46,7 @@ use crate::commands::pip::loggers::{DefaultInstallLogger, DefaultResolveLogger,
 use crate::commands::pip::operations::Modifications;
 use crate::commands::pip::operations::{report_interpreter, report_target_environment};
 use crate::commands::pip::{operations, resolution_markers, resolution_tags};
+use crate::commands::reporters::PythonDownloadReporter;
 use crate::commands::{ExitStatus, diagnostics};
 use crate::printer::Printer;
 
@@ -54,9 +56,11 @@ pub(crate) async fn pip_install(
     requirements: &[RequirementsSource],
     constraints: &[RequirementsSource],
     overrides: &[RequirementsSource],
+    excludes: &[RequirementsSource],
     build_constraints: &[RequirementsSource],
     constraints_from_workspace: Vec<Requirement>,
     overrides_from_workspace: Vec<Requirement>,
+    excludes_from_workspace: Vec<uv_normalize::PackageName>,
     build_constraints_from_workspace: Vec<Requirement>,
     extras: &ExtrasSpecification,
     groups: &GroupsSpecification,
@@ -84,6 +88,8 @@ pub(crate) async fn pip_install(
     modifications: Modifications,
     python_version: Option<PythonVersion>,
     python_platform: Option<TargetTriple>,
+    python_downloads: PythonDownloads,
+    install_mirrors: PythonInstallMirrors,
     strict: bool,
     exclude_newer: ExcludeNewer,
     sources: SourceStrategy,
@@ -118,6 +124,7 @@ pub(crate) async fn pip_install(
         requirements,
         constraints,
         overrides,
+        excludes,
         pylock,
         source_trees,
         groups,
@@ -132,6 +139,7 @@ pub(crate) async fn pip_install(
         requirements,
         constraints,
         overrides,
+        excludes,
         extras,
         Some(groups),
         &client_builder,
@@ -167,6 +175,11 @@ pub(crate) async fn pip_install(
         )
         .collect();
 
+    let excludes: Vec<PackageName> = excludes
+        .into_iter()
+        .chain(excludes_from_workspace)
+        .collect();
+
     // Read build constraints.
     let build_constraints: Vec<NameRequirementSpecification> =
         operations::read_constraints(build_constraints, &client_builder)
@@ -182,16 +195,23 @@ pub(crate) async fn pip_install(
 
     // Detect the current Python interpreter.
     let environment = if target.is_some() || prefix.is_some() {
-        let installation = PythonInstallation::find(
-            &python
-                .as_deref()
-                .map(PythonRequest::parse)
-                .unwrap_or_default(),
+        let python_request = python.as_deref().map(PythonRequest::parse);
+        let reporter = PythonDownloadReporter::single(printer);
+
+        let installation = PythonInstallation::find_or_download(
+            python_request.as_ref(),
             EnvironmentPreference::from_system_flag(system, false),
             python_preference.with_system_flag(system),
+            python_downloads,
+            &client_builder,
             &cache,
+            Some(&reporter),
+            install_mirrors.python_install_mirror.as_deref(),
+            install_mirrors.pypy_install_mirror.as_deref(),
+            install_mirrors.python_downloads_json_url.as_deref(),
             preview,
-        )?;
+        )
+        .await?;
         report_interpreter(&installation, true, printer)?;
         PythonEnvironment::from_installation(installation)
     } else {
@@ -236,18 +256,37 @@ pub(crate) async fn pip_install(
         if break_system_packages {
             debug!("Ignoring externally managed environment due to `--break-system-packages`");
         } else {
-            return if let Some(error) = externally_managed.into_error() {
-                Err(anyhow::anyhow!(
-                    "The interpreter at {} is externally managed, and indicates the following:\n\n{}\n\nConsider creating a virtual environment with `uv venv`.",
+            let managed_message = match externally_managed.into_error() {
+                Some(error) => format!(
+                    "The interpreter at {} is externally managed, and indicates the following:\n\n{}\n",
                     environment.root().user_display().cyan(),
                     textwrap::indent(&error, "  ").green(),
-                ))
-            } else {
-                Err(anyhow::anyhow!(
-                    "The interpreter at {} is externally managed. Instead, create a virtual environment with `uv venv`.",
+                ),
+                None => format!(
+                    "The interpreter at {} is externally managed and cannot be modified.",
                     environment.root().user_display().cyan()
-                ))
+                ),
             };
+
+            let error_message = if system {
+                // Add a hint about the `--system` flag
+                format!(
+                    "{}\n{}{} Virtual environments were not considered due to the `--system` flag",
+                    managed_message,
+                    "hint".bold().cyan(),
+                    ":".bold()
+                )
+            } else {
+                // Add a hint to create a virtual environment
+                format!(
+                    "{}\n{}{} Consider creating a virtual environment, e.g., with `uv venv`",
+                    managed_message,
+                    "hint".bold().cyan(),
+                    ":".bold()
+                )
+            };
+
+            return Err(anyhow::Error::msg(error_message));
         }
     }
 
@@ -531,6 +570,7 @@ pub(crate) async fn pip_install(
             requirements,
             constraints,
             overrides,
+            excludes,
             source_trees,
             project,
             BTreeSet::default(),
