@@ -3,9 +3,10 @@
 
 use std::borrow::BorrowMut;
 use std::ffi::OsString;
+use std::io::Write as _;
 use std::iter::Iterator;
 use std::path::{Path, PathBuf};
-use std::process::{Command, ExitStatus, Output};
+use std::process::{Command, ExitStatus, Output, Stdio};
 use std::str::FromStr;
 use std::{env, io};
 use uv_python::downloads::ManagedPythonDownloadList;
@@ -560,8 +561,8 @@ impl TestContext {
     }
 
     /// Add a custom filter to the `TestContext`.
-    pub fn with_filter(mut self, filter: (String, String)) -> Self {
-        self.filters.push(filter);
+    pub fn with_filter(mut self, filter: (impl Into<String>, impl Into<String>)) -> Self {
+        self.filters.push((filter.0.into(), filter.1.into()));
         self
     }
 
@@ -1104,6 +1105,14 @@ impl TestContext {
         command
     }
 
+    /// Create a `pip debug` command for testing.
+    pub fn pip_debug(&self) -> Command {
+        let mut command = Self::new_command();
+        command.arg("pip").arg("debug");
+        self.add_shared_options(&mut command, true);
+        command
+    }
+
     /// Create a `uv help` command with options shared across scenarios.
     #[allow(clippy::unused_self)]
     pub fn help(&self) -> Command {
@@ -1449,6 +1458,14 @@ impl TestContext {
         command
     }
 
+    /// Create a `uv auth helper --protocol bazel get` command.
+    pub fn auth_helper(&self) -> Command {
+        let mut command = Self::new_command();
+        command.arg("auth").arg("helper");
+        self.add_shared_options(&mut command, false);
+        command
+    }
+
     /// Create a `uv auth token` command.
     pub fn auth_token(&self) -> Command {
         let mut command = Self::new_command();
@@ -1648,6 +1665,7 @@ impl TestContext {
             self.filters(),
             "diff_lock",
             Some(WindowsFilters::Platform),
+            None,
         );
         assert!(status.success(), "{snapshot}");
         let new_lock = fs_err::read_to_string(&lock_path).unwrap();
@@ -1776,7 +1794,9 @@ pub fn python_installations_for_versions(
     python_versions: &[&str],
     download_list: &ManagedPythonDownloadList,
 ) -> anyhow::Result<Vec<PathBuf>> {
-    let cache = Cache::from_path(temp_dir.child("cache").to_path_buf()).init()?;
+    let cache = Cache::from_path(temp_dir.child("cache").to_path_buf())
+        .init_no_wait()?
+        .expect("No cache contention when setting up Python in tests");
     let selected_pythons = python_versions
         .iter()
         .map(|python_version| {
@@ -1790,7 +1810,7 @@ pub fn python_installations_for_versions(
             ) {
                 python.into_interpreter().sys_executable().to_owned()
             } else {
-                panic!("Could not find Python {python_version} for test");
+                panic!("Could not find Python {python_version} for test\nTry `cargo run python install` first, or refer to CONTRIBUTING.md");
             }
         })
         .collect::<Vec<_>>();
@@ -1829,9 +1849,10 @@ pub fn run_and_format<T: AsRef<str>>(
     filters: impl AsRef<[(T, T)]>,
     function_name: &str,
     windows_filters: Option<WindowsFilters>,
+    input: Option<&str>,
 ) -> (String, Output) {
     let (snapshot, output, _) =
-        run_and_format_with_status(command, filters, function_name, windows_filters);
+        run_and_format_with_status(command, filters, function_name, windows_filters, input);
     (snapshot, output)
 }
 
@@ -1844,6 +1865,7 @@ pub fn run_and_format_with_status<T: AsRef<str>>(
     filters: impl AsRef<[(T, T)]>,
     function_name: &str,
     windows_filters: Option<WindowsFilters>,
+    input: Option<&str>,
 ) -> (String, Output, ExitStatus) {
     let program = command
         .borrow_mut()
@@ -1863,10 +1885,30 @@ pub fn run_and_format_with_status<T: AsRef<str>>(
         );
     }
 
-    let output = command
-        .borrow_mut()
-        .output()
-        .unwrap_or_else(|err| panic!("Failed to spawn {program}: {err}"));
+    let output = if let Some(input) = input {
+        let mut child = command
+            .borrow_mut()
+            .stdin(Stdio::piped())
+            .stdout(Stdio::piped())
+            .stderr(Stdio::piped())
+            .spawn()
+            .unwrap_or_else(|err| panic!("Failed to spawn {program}: {err}"));
+        child
+            .stdin
+            .as_mut()
+            .expect("Failed to open stdin")
+            .write_all(input.as_bytes())
+            .expect("Failed to write to stdin");
+
+        child
+            .wait_with_output()
+            .unwrap_or_else(|err| panic!("Failed to read output from {program}: {err}"))
+    } else {
+        command
+            .borrow_mut()
+            .output()
+            .unwrap_or_else(|err| panic!("Failed to spawn {program}: {err}"))
+    };
 
     eprintln!("\n━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━ Unfiltered output ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━");
     eprintln!(
@@ -2065,19 +2107,25 @@ macro_rules! uv_snapshot {
     }};
     ($filters:expr, $spawnable:expr, @$snapshot:literal) => {{
         // Take a reference for backwards compatibility with the vec-expecting insta filters.
-        let (snapshot, output) = $crate::common::run_and_format($spawnable, &$filters, $crate::function_name!(), Some($crate::common::WindowsFilters::Platform));
+        let (snapshot, output) = $crate::common::run_and_format($spawnable, &$filters, $crate::function_name!(), Some($crate::common::WindowsFilters::Platform), None);
+        ::insta::assert_snapshot!(snapshot, @$snapshot);
+        output
+    }};
+    ($filters:expr, $spawnable:expr, input=$input:expr, @$snapshot:literal) => {{
+        // Take a reference for backwards compatibility with the vec-expecting insta filters.
+        let (snapshot, output) = $crate::common::run_and_format($spawnable, &$filters, $crate::function_name!(), Some($crate::common::WindowsFilters::Platform), Some($input));
         ::insta::assert_snapshot!(snapshot, @$snapshot);
         output
     }};
     ($filters:expr, windows_filters=false, $spawnable:expr, @$snapshot:literal) => {{
         // Take a reference for backwards compatibility with the vec-expecting insta filters.
-        let (snapshot, output) = $crate::common::run_and_format($spawnable, &$filters, $crate::function_name!(), None);
+        let (snapshot, output) = $crate::common::run_and_format($spawnable, &$filters, $crate::function_name!(), None, None);
         ::insta::assert_snapshot!(snapshot, @$snapshot);
         output
     }};
     ($filters:expr, universal_windows_filters=true, $spawnable:expr, @$snapshot:literal) => {{
         // Take a reference for backwards compatibility with the vec-expecting insta filters.
-        let (snapshot, output) = $crate::common::run_and_format($spawnable, &$filters, $crate::function_name!(), Some($crate::common::WindowsFilters::Universal));
+        let (snapshot, output) = $crate::common::run_and_format($spawnable, &$filters, $crate::function_name!(), Some($crate::common::WindowsFilters::Universal), None);
         ::insta::assert_snapshot!(snapshot, @$snapshot);
         output
     }};
